package GII;

=head1 NAME

GII - Perl extension to the General Input Interface library (LibGII);

=head1 SYNOPSIS

  use GII;

  $input = new GII::;

  $input->send($event);

  $ev = $input->read;
  $ev = $input->read($mask);
  $input->read(undef, $ev);
  $input->read($mask, $ev);

  $mask = $input->poll($timeout);
  $mask = $input->poll($timeout, $mask);

  $howmany_queued = $input->queued;

  $mask = $input->mask;
  $input->mask($mask);

=head1 DESCRIPTION

GII:: Implements an object class encapsulating the ggi_input_t
type of LibGII.  Methods of this class correspond to the LibGII
API functions.

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA);
use GII::Event;

require DynaLoader;
require AutoLoader;

@ISA = qw(AutoLoader DynaLoader);
$VERSION = '0.03';

bootstrap GII $VERSION;

croak "Cannot Initialize LibGII!  Bailing..." if giiInit();

=head1 Constants

GII::ConfDir contains the name of the configuration file read
by LibGII.

=cut

sub ConfDir { return giiGetConfDir(); }

=head1 Methods

GII:: objects support the following methods:

=head2 new

The object constructor creates a new GII:: object by calling 
LibGII's giiOpen() function and encapsulating the returned 
object.  The first argument, if supplied, designates the 
dynamically loaded input backend library you wish to use by
name, as well as providing any options this library accepts 
in it's "input string".  See the LibGII documentation for more
information.

The second and further options are reserved for backend specific
sub-classes of GII, which may pass additional options to giiOpen
defining special attributes of the input which are too complex
to be conveyed in the "input string".

=cut

sub new {
    my ($class) = shift();
    my ($this) = {};
    if (@_ + 0) {
        $this->{input} = [@_];
        $this->{obj} = giiOpen(@_);
    }
    else {
        $this->{input} = [$ENV{GII_INPUT}, "(default)"];
        $this->{obj} = giiOpen(undef);
    }
    print "my obj = " . $this->{obj} . "\n";
    return undef unless $this->{obj};
    bless $this, $class;
    return $this;
}

sub DESTROY {
    my ($this) = shift();
    $this->giiClose($this->{obj}) unless $this->{reused};
}

=head2 send

The send method will inject the proferred GII::Event object into 
the GII:: object's gii_input.

=cut

sub send {
    my($this) = shift();
    my($ev) = shift();
    return gg_send($this->{obj}, $ev->objptr);
}

=head2 read

Without an argument, waits for (blocks) and returns the least recent 
event received on the gii_input represented by the invoking object.
An optional first argument can be used to set an event mask,
such that events that do not match the mask are left untouched 
and not returned.  The returned object is a new GII::Event object,
and has been subclassed into the correct GII::Event subclass.
The function may also return undef if there was an error.

A second argument allows you to reuse a GII::Event, the new event
is placed in this object.  In this case, using a first argument of
undef() will force a pass-all event mask.  Note you should take
care when re-using events; it is assumed you know what you are
doing if you try this.  The return value is the same as above in this
case.

Event mask constants may be imported by explicitly calling use()
on GII::Event, or they may be referred to by fully qualified
identifiers (they are in the GII:: namespace as well as the GII::Event::
namespace.)

=cut

sub read {
    my($this) = shift();
    my($mask) = shift();
    $mask = &GII::emAll unless defined $mask;
    my($ev) = (@_ + 0) ? shift() : new GII::Event;
    return undef unless $this->gg_read($this->{obj}, $ev->{obj}, $mask);
    $ev->{reused} = 1;
    $ev = new GII::Event $ev->type, $ev->objptr;
    return $ev;
}

=head2 poll

Without an argument, returns a mask telling which types of events 
are waiting to be read from the input.

An optional first argument can be a perl scalar representing
a maximum number of microseconds to wait.  (IMPLEMENTATION
NOTE: will eventually add timeval and Time:: objects to acceptable 
parameters.)

An optional second argument can be used to set an event mask,
such that events that do not match the mask are not checked for
or represented in the return value.

Event mask constants may be imported by explicitly calling use()
on GII::Event, or they may be referred to by fully qualified
identifiers (they are in the GII:: namespace as well as the GII::Event::
namespace.)

=cut

sub poll {
    my($this) = $_[0];
    my($mask) = $_[2];
    $mask = GII::emAll() unless defined $mask;
    return $this->gg_poll_msec($this->{obj}, $mask, $_[1]);
}

=head2 queued

Returns the number of events waiting to be read on the input.

=cut

sub queued {
    my($this) = shift();
    return($this->queued($this->{obj}));
}

=head2 mask

Without an argument, gets the mask which is filtering events on
the input (events not matching the mask are discarded from the
input or simply not generated in the first place.)

With an argument, sets the mask to the given value, returning
an error code or zero (GGI_OK) for success.

=cut

sub mask {
    my($this) = shift();
    if (@_ + 0) {
	return $this->gg_set_mask($this->{obj}, shift());
    }
    return $this->gg_get_mask($this->{obj});
}

END {
    croak "Error De-Initializing LibGII!" if giiExit();
}

1;
__END__


=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GGI::(3pm), GII::Event(3pm), perl(1).

=cut
